from .rule import BaseRule
from .config import DispatcherConfig

import typing
import logging

logger = logging.getLogger(__name__)


class SkipHandler(Exception):
    pass


class Handler:
    def __init__(
        self, event_type: DispatcherConfig.event_enum, handler: typing.Callable, rules: typing.List[BaseRule]
    ):
        """

        :param event_type: type of event which this handler accepted
        :param handler: coroutine
        :param rules: list of rules which be executed
        """
        self.event_type: DispatcherConfig.event_enum = event_type
        self.handler: typing.Callable = handler
        self.rules: typing.List[BaseRule] = rules

    async def execute_handler(self, *args) -> bool:
        """
        Execute rules and handler

        If all perfect, returns True.
        :param args:
        :return:
        """
        # args - (event, data)
        if self.rules:
            _execute = False
            for rule in self.rules:
                result = await rule(*args)
                if not result:
                    _execute = False
                    break
                _execute = True

            if _execute:
                await self.handler(*args)
                return True

        else:
            await self.handler(*args)
            return True
